--- old/usr/src/uts/common/fs/zfs/vdev_queue.c	2013-11-21 00:19:08.413067486 +0000
+++ new/usr/src/uts/common/fs/zfs/vdev_queue.c	2013-11-21 00:19:08.418746889 +0000
@@ -25,6 +25,7 @@
 
 /*
  * Copyright (c) 2013 by Delphix. All rights reserved.
+ * Copyright (c) 2013 Steven Hartland. All rights reserved.
  */
 
 #include <sys/zfs_context.h>
@@ -229,6 +230,8 @@
 		    vdev_queue_offset_compare,
 		    sizeof (zio_t), offsetof(struct zio, io_queue_node));
 	}
+
+	vq->vq_last_queued_offset = 0;
 }
 
 void
@@ -725,3 +728,33 @@
 
 	mutex_exit(&vq->vq_lock);
 }
+
+/*
+ * As these three methods are only used for load calculations we're not
+ * concerned if we get an incorrect value on 32bit platforms due to lack of
+ * vq_lock mutex use here, instead we prefer to keep it lock free for
+ * performance.
+ */
+int
+vdev_queue_length(vdev_t *vd)
+{
+	vdev_queue_t *vq = &vd->vdev_queue;
+	int len = 0;
+
+	for (zio_priority_t p = 0; p < ZIO_PRIORITY_NUM_QUEUEABLE; p++)
+		len += avl_numnodes(&vq->vq_class[p].vqc_queued_tree);
+
+	return (len);
+}
+
+uint64_t
+vdev_queue_last_queued_offset(vdev_t *vd)
+{
+	return (vd->vdev_queue.vq_last_queued_offset);
+}
+
+void
+vdev_queue_register_last_queued_offset(vdev_t *vd, zio_t *zio)
+{
+	vd->vdev_queue.vq_last_queued_offset = zio->io_offset + zio->io_size;
+}
